#include <stdio.h>
#include <stdlib.h>
#include <openssl/rand.h>
#include <math.h>
//#include <openssl/rc4.h>
#include <openssl/hmac.h>
#include <string.h>
//Structure used to implement shared secrets
struct SSecret{
	unsigned char key[32];
	unsigned char bridgeFingerprint[512];
	int epoch;
	int timePeriodDuration; //TODO:Use this for syncing between bridge/BA and bridge/client
};

typedef struct SSecret SharedSecret;

//hmac takes a key, keylength secret and secret length. The last parameter is a 32 byte output for the keyed secret hash
int  hmac(unsigned char key[], size_t keylen, unsigned char sharedSecret[], int secretSize, unsigned char out[]) {
	int           i;
	HMAC_CTX      ctx;
	unsigned int  len;
	HMAC_Init(&ctx, key, keylen, EVP_sha256(  ));
	HMAC_Update(&ctx, sharedSecret, secretSize);
	HMAC_Final(&ctx, out, &len);
	HMAC_cleanup(&ctx); 
	return 0;
}

//generateSPA creates an SPA based on a sharedsecret and cleartext value
bool generateMACKey(SharedSecret ss, unsigned char cleartext[], size_t cleartextSize, unsigned char SPA[32])
{
  unsigned char SHA256Val[32];
  hmac(ss.key,32,cleartext,cleartextSize,SHA256Val);
  for(int i=0;i<32;i++)
    SPA[i]=SHA256Val[i];
  return true;
}


//generateSPA creates an SPA based on a sharedsecret and cleartext value
bool generateSPA(SharedSecret ss, unsigned char cleartext[], size_t cleartextSize, unsigned char SPA[4])
{
	unsigned char SHA256Val[32];
	hmac(ss.key,32,cleartext,cleartextSize,SHA256Val);
	for(int i=0;i<4;i++)
		SPA[i]=SHA256Val[i];
	return true;
}
//verifySPA checks if an SPA value exists for a key in the current time period.
bool verifySPA(unsigned char spa[4],unsigned char cleartext[],size_t cleartextLen, SharedSecret ss)
{
	unsigned char sha256Val[32];
	hmac(ss.key,32,cleartext,cleartextLen,sha256Val);
	for(int j=0;j<4;j++)
	{
		printf("%d",sha256Val[j]);
	}
	if(	sha256Val[0]==spa[0] && 
		sha256Val[1]==spa[1] && 
		sha256Val[2]==spa[2] && 
		sha256Val[3]==spa[3]
	)
	{
		printf("\nMatch found!\n");
		return true;
	}
	return false;
}

//generateSharedSecret makes up a 256 bit random number for the BA/bridge shared secret.
int generateSharedSecret(unsigned char secret[32]) 
{
	if(!(RAND_pseudo_bytes(secret, 32)))
	return 1;
	return 0;
}

