/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Copyright 2011 Rob Smits, Divam Jain, Sarah Pidcock
 */

// Standard C++ libraries
#include <iostream>
#include <iomanip>
#include <sstream>
#include <map>

// Misc C libraries
#include <time.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <pthread.h>

//BridgeSPA includes
#include "crypto.c"
#include "utils.cpp"

using namespace std;


int main(int argc, char** argv) {
  if (argc < 3) {
    cout << "Usage: " << argv[0] << " <SeedKey> <update frequency>" << endl;
    return 0;
  }

  char* str_seed_key = argv[1];
  SharedSecret bridgeSecret;
  stringstream ss (stringstream::in | stringstream::out);
  string mac_data;
  int update_frequency = 0;
  unsigned char mac_key[32];
  time_t epoch_index;  

  int seed_key_len = strlen(str_seed_key);
  unsigned char *seed_key = (unsigned char*) malloc(sizeof(unsigned char) * seed_key_len);
  hex_to_binary(str_seed_key, seed_key, &seed_key_len);

  memset(bridgeSecret.key, 0, sizeof(bridgeSecret.key));
  copyKey(seed_key, seed_key_len, bridgeSecret.key);
  
  ss << argv[2];
  ss >> update_frequency;

  if (update_frequency <= 0) {
    cout << "Invalid update frequency" << endl;
    return 0;
  }

  epoch_index = time(NULL) / update_frequency;

  cout << "epoch_index: " << epoch_index << endl;
  
  ss.clear();
  ss.str(std::string());
  
  ss << epoch_index;
 
  mac_data = ss.str(); 

  cout << "Using key: " << hex << hex_binary_to_str(bridgeSecret.key, 32) << dec << endl;
  cout << "Using MAC data (epoch index): " << mac_data << endl;

  generateMACKey(bridgeSecret, (unsigned char*)mac_data.c_str(), mac_data.size(), mac_key);

  ss.clear();
  ss.str(std::string());
  
  cout << "MACKey: " << hex_binary_to_str(mac_key, 32) << endl;;
  
  free(seed_key);
}
